@@ -9,10 +9,15 @@ __all__ = (
     'CheckboxInput', 'Select', 'NullBooleanSelect', 'SelectMultiple',
     'RadioSelect', 'CheckboxSelectMultiple', 'SearchInput', 'RangeInput',
     'ColorInput', 'EmailInput', 'URLInput', 'PhoneNumberInput', 'NumberInput',
-    'IPAddressInput',
+    'IPAddressInput', 'MultiWidget', 'Widget', 'SplitDateTimeWidget',
+    'SplitHiddenDateTimeWidget', 'MultipleHiddenInput',
 )
 
 
+class Widget(forms.Widget):
+    pass
+
+
 class Input(forms.TextInput):
     input_type = None
     template_name = 'floppyforms/input.html'
@@ -89,7 +94,8 @@ class SlugInput(TextInput):
 class IPAddressInput(TextInput):
 
     def get_context_data(self):
-        self.attrs['pattern'] = "(25[0-5]|2[0-4]\d|[0-1]?\d?\d)(\.(25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}"
+        self.attrs['pattern'] = ("(25[0-5]|2[0-4]\d|[0-1]?\d?\d)(\.(25[0-5]|"
+                                 "2[0-4]\d|[0-1]?\d?\d)){3}")
         return super(IPAddressInput, self).get_context_data()
 
 
@@ -245,3 +251,57 @@ class RadioSelect(forms.RadioSelect, Select):
 
     def render(self, name, value, attrs=None, choices=()):
         return Select.render(self, name, value, attrs=attrs, choices=choices)
+
+
+class MultiWidget(forms.MultiWidget):
+    pass
+
+
+class SplitDateTimeWidget(MultiWidget):
+
+    def __init__(self, attrs=None, date_format=None, time_format=None):
+        widgets = (DateInput(attrs=attrs, format=date_format),
+                   TimeInput(attrs=attrs, format=time_format))
+        super(SplitDateTimeWidget, self).__init__(widgets, attrs)
+
+    def decompress(self, value):
+        if value:
+            return [value.date(), value.time().replace(microseconds=0)]
+        return [None, None]
+
+
+class SplitHiddenDateTimeWidget(SplitDateTimeWidget):
+    is_hidden = True
+
+    def __init__(self, attrs=None, date_format=None, time_format=None):
+        super(SplitHiddenDateTimeWidget, self).__init__(attrs, date_format,
+                                                        time_format)
+        for widget in self.widgets:
+            widget.input_type = 'hidden'
+            widget.is_hidden = True
+
+
+class MultipleHiddenInput(HiddenInput):
+    """<input type="hidden"> for fields that have a list of values"""
+    def __init__(self, attrs=None, choices=()):
+        super(MultipleHiddenInput, self).__init__(attrs)
+        self.choices = choices
+
+    def render(self, name, value, attrs=None, choices=()):
+        if value is None:
+            value = []
+
+        final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
+        id_ = final_attrs.get('id', None)
+        inputs = []
+        for i, v in enumerate(value):
+            input_attrs = dict(value=force_unicode(v), **final_attrs)
+            if id_:
+                input_attrs['id'] = '%s_%s' % (id_, i)
+            del input_attrs['type']
+            del input_attrs['value']
+            input_ = HiddenInput()
+            input_.is_required = self.is_required
+            inputs.append(input_.render(name, force_unicode(v), input_attrs))
+        return "\n".join(inputs)
+
